# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["Operation", "Result"]


class Result(BaseModel):
    id: Optional[int] = None
    """Operation Id"""

    completed: Optional[str] = None
    """Operation end time"""

    incoming_messages: Optional[str] = FieldInfo(alias="incomingMessages", default=None)
    """List of messages received from external APIs during operation processing"""

    operation: Optional[
        Literal[
            "CREATE_PAYMENT",
            "CHECKOUT",
            "CANCEL",
            "CONFIRMATION",
            "CASCADING",
            "REDIRECT",
            "CONTINUE",
            "CONTINUE_ANTI_FRAUD",
            "DETECT_FRAUD",
            "DEPOSIT",
            "WITHDRAWAL",
            "REFUND",
            "CHARGEBACK",
            "CHECK_STATE",
            "HANDLE_WEBHOOK",
            "MANUAL_UPDATE",
        ]
    ] = None
    """Operation performed during payment processing"""

    outgoing_messages: Optional[str] = FieldInfo(alias="outgoingMessages", default=None)
    """List of messages sent to external APIs during operation processing"""

    payment_state: Optional[Literal["CHECKOUT", "PENDING", "CANCELLED", "DECLINED", "COMPLETED"]] = FieldInfo(
        alias="paymentState", default=None
    )
    """Payment State"""

    started: Optional[str] = None
    """Operation start time"""


class Operation(BaseModel):
    result: Optional[List[Result]] = None

    status: Optional[int] = None
    """HTTP status code"""

    timestamp: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Payment", "Result", "ResultBillingAddress", "ResultCustomer", "ResultPaymentMethodDetails"]


class ResultBillingAddress(BaseModel):
    address_line1: Optional[str] = FieldInfo(alias="addressLine1", default=None)
    """Line 1 of the address (e.g., Number, street, etc)"""

    address_line2: Optional[str] = FieldInfo(alias="addressLine2", default=None)
    """Line 2 of the address (e.g., Suite, apt)"""

    city: Optional[str] = None
    """City name"""

    country_code: Optional[str] = FieldInfo(alias="countryCode", default=None)
    """2-character IS0-3166-1 country code"""

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """Postal code"""

    state: Optional[str] = None
    """State code"""


class ResultCustomer(BaseModel):
    account_name: Optional[str] = FieldInfo(alias="accountName", default=None)
    """Customer account name in the provider's system.

    Used for some types of withdrawals.
    """

    account_number: Optional[str] = FieldInfo(alias="accountNumber", default=None)
    """Customer account number in the provider's system.

    Used for some types of withdrawals.
    """

    bank: Optional[str] = None
    """Customer bank. Used for some types of withdrawals."""

    bank_branch: Optional[str] = FieldInfo(alias="bankBranch", default=None)
    """Customer bank branch. Used for some types of withdrawals."""

    citizenship_country_code: Optional[str] = FieldInfo(alias="citizenshipCountryCode", default=None)
    """Customer country of citizenship"""

    date_of_birth: Optional[str] = FieldInfo(alias="dateOfBirth", default=None)

    date_of_first_deposit: Optional[str] = FieldInfo(alias="dateOfFirstDeposit", default=None)
    """Date of the first deposit from the customer"""

    deposits_amount: Optional[int] = FieldInfo(alias="depositsAmount", default=None)
    """How much the customer has deposited, in base currency"""

    deposits_cnt: Optional[int] = FieldInfo(alias="depositsCnt", default=None)
    """How many times the customer made a deposit"""

    document_number: Optional[str] = FieldInfo(alias="documentNumber", default=None)
    """An identifier for the customer assigned by a government authority"""

    document_type: Optional[
        Literal[
            "AR_CDI",
            "AR_CUIL",
            "AR_CUIT",
            "AR_DNI",
            "AR_OTRO",
            "BR_CNPJ",
            "BR_CPF",
            "CL_OTRO",
            "CL_RUN",
            "CL_RUT",
            "CO_CC",
            "CO_CE",
            "CO_DL",
            "CO_DNI",
            "CO_NE",
            "CO_NIT",
            "CO_PP",
            "CO_SS",
            "CO_TI",
            "CR_CDI",
            "EC_DNI",
            "EC_PP",
            "EC_RUC",
            "GT_CUI",
            "GT_DPI",
            "GT_NIT",
            "MX_CURP",
            "MX_IFE",
            "MX_PP",
            "MX_RFC",
            "PA_CIP",
            "PE_CE",
            "PE_DNI",
            "PE_OTRO",
            "PE_PP",
            "PE_RUC",
        ]
    ] = FieldInfo(alias="documentType", default=None)
    """Document Type"""

    email: Optional[str] = None
    """Email address of the customer"""

    first_name: Optional[str] = FieldInfo(alias="firstName", default=None)

    kyc_status: Optional[bool] = FieldInfo(alias="kycStatus", default=None)
    """Indicates whether the customer has passed KYC verification"""

    last_name: Optional[str] = FieldInfo(alias="lastName", default=None)

    locale: Optional[str] = None
    """Customer preferred display language"""

    payment_instrument_kyc_status: Optional[bool] = FieldInfo(alias="paymentInstrumentKycStatus", default=None)
    """
    Indicates whether the payment instrument (usually the card number) has passed
    KYC verification
    """

    phone: Optional[str] = None
    """International phone number of the customer, without the '+'.

    Use a space as a separator between the dialing country code and local phone
    number.
    """

    reference_id: Optional[str] = FieldInfo(alias="referenceId", default=None)
    """Id of the customer assigned by Merchant"""

    routing_group: Optional[str] = FieldInfo(alias="routingGroup", default=None)
    """Identify the customer as belonging to a specific group that is used for routing"""

    withdrawals_amount: Optional[int] = FieldInfo(alias="withdrawalsAmount", default=None)
    """How much the customer has withdrawn, in base currency"""

    withdrawals_cnt: Optional[int] = FieldInfo(alias="withdrawalsCnt", default=None)
    """How many times the customer made a withdrawal"""


class ResultPaymentMethodDetails(BaseModel):
    card_expiry_month: Optional[str] = FieldInfo(alias="cardExpiryMonth", default=None)
    """Card expiration month (for BASIC_CARD payment method only)"""

    card_expiry_year: Optional[str] = FieldInfo(alias="cardExpiryYear", default=None)
    """Card expiration year (for BASIC_CARD payment method only)"""

    cardholder_name: Optional[str] = FieldInfo(alias="cardholderName", default=None)
    """Cardholder name (for BASIC_CARD payment method only)"""

    card_issuing_country_code: Optional[str] = FieldInfo(alias="cardIssuingCountryCode", default=None)
    """Card issuing country code (for BASIC_CARD payment method only)"""

    customer_account_number: Optional[str] = FieldInfo(alias="customerAccountNumber", default=None)
    """Customer account Id in external system or masked card PAN"""


class Result(BaseModel):
    id: Optional[str] = None
    """Payment Id"""

    amount: Optional[float] = None
    """Amount sent to the payment provider"""

    billing_address: Optional[ResultBillingAddress] = FieldInfo(alias="billingAddress", default=None)
    """Customer's billing address"""

    currency: Optional[str] = None
    """Currency sent to the payment provider"""

    customer: Optional[ResultCustomer] = None

    customer_amount: Optional[float] = FieldInfo(alias="customerAmount", default=None)
    """Amount from payment request.

    Filled only if the request currency differs from the currency sent to the
    payment provider.
    """

    customer_currency: Optional[str] = FieldInfo(alias="customerCurrency", default=None)
    """Currency from payment request.

    Filled only if it differs from the currency sent to the payment provider.
    """

    description: Optional[str] = None
    """Description of the transaction"""

    error_code: Optional[str] = FieldInfo(alias="errorCode", default=None)
    """Check 'Error Codes' section for details"""

    external_fee_amount: Optional[float] = FieldInfo(alias="externalFeeAmount", default=None)
    """Provider fee. Filled only if supported by the provider."""

    external_fee_currency: Optional[str] = FieldInfo(alias="externalFeeCurrency", default=None)
    """Provider fee currency. Filled only if supported by the provider."""

    external_result_code: Optional[str] = FieldInfo(alias="externalResultCode", default=None)
    """Result code from external provider"""

    parent_payment_id: Optional[str] = FieldInfo(alias="parentPaymentId", default=None)
    """Initial transaction Id from payment request"""

    payment_method: Optional[
        Literal[
            "BASIC_CARD",
            "CRYPTO",
            "FLEXEPIN",
            "MACROPAY",
            "SKRILL",
            "PAYRETAILERS",
            "LOCALPAYMENT",
            "MONNET",
            "PAYPAL",
            "NETELLER",
            "TRUSTPAYMENTS",
            "PAYMAXIS",
            "GATE8TRANSACT",
            "TINK",
            "B2BINPAY",
            "CLICK",
            "MONETIX",
            "PERFECTMONEY",
            "VOLT",
            "KESSPAY",
            "BILLLINE",
            "NGENIUS",
            "ASTROPAY",
            "ALYCEPAY",
            "PIX",
            "BOLETO",
            "UPI",
            "PAYTM",
            "NETBANKING",
            "FINRAX",
            "SPOYNT",
            "XINPAY",
            "OMNIMATRIX",
            "DPOPAY",
            "EXTERNAL_HPP",
            "XANPAY",
            "INRPAY",
            "ARI10",
            "SOFORT",
            "GIROPAY",
            "PAYSAFECARD",
            "PAYSAFECASH",
            "OPEN_BANKING",
            "KLARNA",
            "SPEI",
            "PAYCASH",
            "RAPIPAGO",
            "PAGOFACIL",
            "RAPIDTRANSFER",
            "MOBILE_MONEY",
            "INTERAC",
            "INTERAC_ETO",
            "INTERAC_RTO",
            "INTERAC_ACH",
            "PICPAY",
            "MOLLIE",
            "TED",
            "ZIPAY",
            "PSE",
            "EFECTY",
            "BANKTRANSFER",
            "PEC",
            "OXXO",
            "WEBPAY",
            "PAGOEFECTIVO",
            "MIFINITY",
            "PAYPORT",
            "JETONCASH",
            "JETONWALLET",
            "NODA",
            "NODA_REVOLUT",
            "ALFAKIT",
            "PAYFUN",
            "EMANAT",
            "M10",
            "RUBPAY",
            "MONERCHY",
            "MUCHBETTER",
            "YAPILY",
            "INAI",
            "IMPS",
            "RTGS",
            "PAYID",
            "ZOTAPAY",
            "SBP",
            "P2P_CARD",
            "P2P_IBAN",
            "P2P_SBP",
            "P2P_MOBILE",
            "PUSH",
            "GATEIQ",
            "VIETTEL",
            "ZALO",
            "QR",
            "CUP",
            "CODI",
            "PAY2PLAY",
            "BKASH",
            "NAGAD",
            "ROCKET",
            "VIRTUAL_ACCOUNT",
            "MULTIBANCO",
            "BLIK",
            "MBWAY",
            "P24",
            "MISTERCASH",
            "MACH",
            "KHIPU",
            "NEFT",
            "STICPAY",
            "SBERPAY",
            "MOBILE_COMMERCE",
            "BINANCE_PAY",
            "MPAY",
            "CHEK",
            "KLAP_EFECTIVO",
            "KLAP_TRANSFERENCIA",
            "PAGO46",
            "HITES",
            "SERVIFACIL",
            "OPENPAYD",
            "FAWRY",
            "EPS",
            "IDEAL",
            "TRUSTLY",
            "USSD",
            "MPESA",
            "ENAIRA",
            "ONEVOUCHER",
            "BANCONTACT",
            "SWISH",
            "EFT",
            "GCASH",
            "PAYMAYA",
            "PAGO_MOVIL",
            "PAGO_MOVIL_INST",
            "BIOPAGO",
            "CASH",
            "VOUCHERRY",
            "APPLEPAY",
            "GOOGLEPAY",
            "BRITE",
            "VOUCHSTAR",
            "REVOLUT",
            "ONLINE_BANKING",
            "PROMPTPAY",
            "TRUEMONEY",
            "MOMOPAY_VN",
            "MOMOPAY_RW",
            "VNPAY_QR",
            "N26",
            "WISE",
            "PAYDO_WALLET",
            "PAPARA",
            "PAYOUT_SEPA_BATCH",
            "PAYOUT_NONSEPA_REQUEST",
        ]
    ] = FieldInfo(alias="paymentMethod", default=None)
    """Payment Method"""

    payment_method_details: Optional[ResultPaymentMethodDetails] = FieldInfo(alias="paymentMethodDetails", default=None)

    payment_type: Optional[Literal["DEPOSIT", "WITHDRAWAL", "REFUND"]] = FieldInfo(alias="paymentType", default=None)
    """Payment Type"""

    recurring_token: Optional[str] = FieldInfo(alias="recurringToken", default=None)
    """Token that can be used to continue the recurring chain"""

    redirect_url: Optional[str] = FieldInfo(alias="redirectUrl", default=None)
    """URL to redirect the customer"""

    reference_id: Optional[str] = FieldInfo(alias="referenceId", default=None)
    """referenceId from payment request"""

    start_recurring: Optional[bool] = FieldInfo(alias="startRecurring", default=None)
    """Indicates whether this payment has started a recurring chain"""

    state: Optional[Literal["CHECKOUT", "PENDING", "CANCELLED", "DECLINED", "COMPLETED"]] = None
    """Payment State"""

    terminal_name: Optional[str] = FieldInfo(alias="terminalName", default=None)
    """The name of the provider that was used to process this payment"""


class Payment(BaseModel):
    result: Optional[Result] = None

    status: Optional[int] = None
    """HTTP status code"""

    timestamp: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import subscription_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.subscription import Subscription

__all__ = ["SubscriptionsResource", "AsyncSubscriptionsResource"]


class SubscriptionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TralahM/paymaxis-python#accessing-raw-response-data-eg-headers
        """
        return SubscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TralahM/paymaxis-python#with_streaming_response
        """
        return SubscriptionsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """
        Find Subscription by Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/v1/subscriptions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    def update(
        self,
        id: str,
        *,
        state: Literal["CANCELLED"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """
        Patch Subscription

        Args:
          state: New subscription state

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/v1/subscriptions/{id}",
            body=maybe_transform({"state": state}, subscription_update_params.SubscriptionUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )


class AsyncSubscriptionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TralahM/paymaxis-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TralahM/paymaxis-python#with_streaming_response
        """
        return AsyncSubscriptionsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """
        Find Subscription by Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/v1/subscriptions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    async def update(
        self,
        id: str,
        *,
        state: Literal["CANCELLED"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """
        Patch Subscription

        Args:
          state: New subscription state

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/v1/subscriptions/{id}",
            body=await async_maybe_transform({"state": state}, subscription_update_params.SubscriptionUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )


class SubscriptionsResourceWithRawResponse:
    def __init__(self, subscriptions: SubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.retrieve = to_raw_response_wrapper(
            subscriptions.retrieve,
        )
        self.update = to_raw_response_wrapper(
            subscriptions.update,
        )


class AsyncSubscriptionsResourceWithRawResponse:
    def __init__(self, subscriptions: AsyncSubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.retrieve = async_to_raw_response_wrapper(
            subscriptions.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            subscriptions.update,
        )


class SubscriptionsResourceWithStreamingResponse:
    def __init__(self, subscriptions: SubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.retrieve = to_streamed_response_wrapper(
            subscriptions.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            subscriptions.update,
        )


class AsyncSubscriptionsResourceWithStreamingResponse:
    def __init__(self, subscriptions: AsyncSubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.retrieve = async_to_streamed_response_wrapper(
            subscriptions.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            subscriptions.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal

import httpx

from ...types import payment_list_params, payment_create_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from .operations import (
    OperationsResource,
    AsyncOperationsResource,
    OperationsResourceWithRawResponse,
    AsyncOperationsResourceWithRawResponse,
    OperationsResourceWithStreamingResponse,
    AsyncOperationsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.payment import Payment
from ...types.payment_list_response import PaymentListResponse

__all__ = ["PaymentsResource", "AsyncPaymentsResource"]


class PaymentsResource(SyncAPIResource):
    @cached_property
    def operations(self) -> OperationsResource:
        return OperationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> PaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TralahM/paymaxis-python#accessing-raw-response-data-eg-headers
        """
        return PaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TralahM/paymaxis-python#with_streaming_response
        """
        return PaymentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        currency: str,
        payment_type: Literal["DEPOSIT", "WITHDRAWAL", "REFUND"],
        additional_parameters: Dict[str, str] | NotGiven = NOT_GIVEN,
        amount: float | NotGiven = NOT_GIVEN,
        billing_address: payment_create_params.BillingAddress | NotGiven = NOT_GIVEN,
        card: payment_create_params.Card | NotGiven = NOT_GIVEN,
        customer: payment_create_params.Customer | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        parent_payment_id: str | NotGiven = NOT_GIVEN,
        payment_method: Literal[
            "BASIC_CARD",
            "CRYPTO",
            "FLEXEPIN",
            "MACROPAY",
            "SKRILL",
            "PAYRETAILERS",
            "LOCALPAYMENT",
            "MONNET",
            "PAYPAL",
            "NETELLER",
            "TRUSTPAYMENTS",
            "PAYMAXIS",
            "GATE8TRANSACT",
            "TINK",
            "B2BINPAY",
            "CLICK",
            "MONETIX",
            "PERFECTMONEY",
            "VOLT",
            "KESSPAY",
            "BILLLINE",
            "NGENIUS",
            "ASTROPAY",
            "ALYCEPAY",
            "PIX",
            "BOLETO",
            "UPI",
            "PAYTM",
            "NETBANKING",
            "FINRAX",
            "SPOYNT",
            "XINPAY",
            "OMNIMATRIX",
            "DPOPAY",
            "EXTERNAL_HPP",
            "XANPAY",
            "INRPAY",
            "ARI10",
            "SOFORT",
            "GIROPAY",
            "PAYSAFECARD",
            "PAYSAFECASH",
            "OPEN_BANKING",
            "KLARNA",
            "SPEI",
            "PAYCASH",
            "RAPIPAGO",
            "PAGOFACIL",
            "RAPIDTRANSFER",
            "MOBILE_MONEY",
            "INTERAC",
            "INTERAC_ETO",
            "INTERAC_RTO",
            "INTERAC_ACH",
            "PICPAY",
            "MOLLIE",
            "TED",
            "ZIPAY",
            "PSE",
            "EFECTY",
            "BANKTRANSFER",
            "PEC",
            "OXXO",
            "WEBPAY",
            "PAGOEFECTIVO",
            "MIFINITY",
            "PAYPORT",
            "JETONCASH",
            "JETONWALLET",
            "NODA",
            "NODA_REVOLUT",
            "ALFAKIT",
            "PAYFUN",
            "EMANAT",
            "M10",
            "RUBPAY",
            "MONERCHY",
            "MUCHBETTER",
            "YAPILY",
            "INAI",
            "IMPS",
            "RTGS",
            "PAYID",
            "ZOTAPAY",
            "SBP",
            "P2P_CARD",
            "P2P_IBAN",
            "P2P_SBP",
            "P2P_MOBILE",
            "PUSH",
            "GATEIQ",
            "VIETTEL",
            "ZALO",
            "QR",
            "CUP",
            "CODI",
            "PAY2PLAY",
            "BKASH",
            "NAGAD",
            "ROCKET",
            "VIRTUAL_ACCOUNT",
            "MULTIBANCO",
            "BLIK",
            "MBWAY",
            "P24",
            "MISTERCASH",
            "MACH",
            "KHIPU",
            "NEFT",
            "STICPAY",
            "SBERPAY",
            "MOBILE_COMMERCE",
            "BINANCE_PAY",
            "MPAY",
            "CHEK",
            "KLAP_EFECTIVO",
            "KLAP_TRANSFERENCIA",
            "PAGO46",
            "HITES",
            "SERVIFACIL",
            "OPENPAYD",
            "FAWRY",
            "EPS",
            "IDEAL",
            "TRUSTLY",
            "USSD",
            "MPESA",
            "ENAIRA",
            "ONEVOUCHER",
            "BANCONTACT",
            "SWISH",
            "EFT",
            "GCASH",
            "PAYMAYA",
            "PAGO_MOVIL",
            "PAGO_MOVIL_INST",
            "BIOPAGO",
            "CASH",
            "VOUCHERRY",
            "APPLEPAY",
            "GOOGLEPAY",
            "BRITE",
            "VOUCHSTAR",
            "REVOLUT",
            "ONLINE_BANKING",
            "PROMPTPAY",
            "TRUEMONEY",
            "MOMOPAY_VN",
            "MOMOPAY_RW",
            "VNPAY_QR",
            "N26",
            "WISE",
            "PAYDO_WALLET",
            "PAPARA",
            "PAYOUT_SEPA_BATCH",
            "PAYOUT_NONSEPA_REQUEST",
        ]
        | NotGiven = NOT_GIVEN,
        recurring_token: str | NotGiven = NOT_GIVEN,
        reference_id: str | NotGiven = NOT_GIVEN,
        return_url: str | NotGiven = NOT_GIVEN,
        start_recurring: bool | NotGiven = NOT_GIVEN,
        subscription: payment_create_params.Subscription | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Payment:
        """
        Payment request, used for DEPOSITS, WITHDRAWALS and REFUNDS

        Args:
          currency: Payment currency

          payment_type: Payment Type

          additional_parameters: Additional parameters required by some payment providers. Contact support for
              more information.

          amount: Payment amount

          billing_address: Customer's billing address

          card: You must be PCI DSS compliant to collect card data on your side. If you are not
              certified, do not add this field to your request and we will collect the data on
              our page.

          description: Description of the transaction shown to the Customer. Can be sent outside the
              system.

          parent_payment_id: Id of initial deposit for refunds, Id of initial recurring payment for
              subsequent payments

          payment_method: Payment Method

          recurring_token: To continue recurring chain, send a token from a previously initiated recurring
              payment.

          reference_id: Reference assigned by Merchant. Will not go outside the system. Will be sent
              unchanged in the PaymentResponse.

          return_url: URL to redirect Customer after processing

          start_recurring: Send 'true' if you want this payment to initiate recurring chain. Default is
              'false'.

          subscription: Subscription to bill customers at regular intervals. Used only with
              'startRecurring=true'.

          webhook_url: Url to receive payment status notifications

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/payments",
            body=maybe_transform(
                {
                    "currency": currency,
                    "payment_type": payment_type,
                    "additional_parameters": additional_parameters,
                    "amount": amount,
                    "billing_address": billing_address,
                    "card": card,
                    "customer": customer,
                    "description": description,
                    "parent_payment_id": parent_payment_id,
                    "payment_method": payment_method,
                    "recurring_token": recurring_token,
                    "reference_id": reference_id,
                    "return_url": return_url,
                    "start_recurring": start_recurring,
                    "subscription": subscription,
                    "webhook_url": webhook_url,
                },
                payment_create_params.PaymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Payment,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Payment:
        """
        Find Payment by Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/v1/payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Payment,
        )

    def list(
        self,
        *,
        created: payment_list_params.Created | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        updated: payment_list_params.Updated | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PaymentListResponse:
        """
        Get a list of payments sorted by creation date (most recent first)

        Args:
          limit: The numbers of items to return. Default is 50.

          offset: The number of items to skip before starting to collect the result set. Default
              is 0.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/v1/payments",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created": created,
                        "limit": limit,
                        "offset": offset,
                        "updated": updated,
                    },
                    payment_list_params.PaymentListParams,
                ),
            ),
            cast_to=PaymentListResponse,
        )


class AsyncPaymentsResource(AsyncAPIResource):
    @cached_property
    def operations(self) -> AsyncOperationsResource:
        return AsyncOperationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/TralahM/paymaxis-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/TralahM/paymaxis-python#with_streaming_response
        """
        return AsyncPaymentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        currency: str,
        payment_type: Literal["DEPOSIT", "WITHDRAWAL", "REFUND"],
        additional_parameters: Dict[str, str] | NotGiven = NOT_GIVEN,
        amount: float | NotGiven = NOT_GIVEN,
        billing_address: payment_create_params.BillingAddress | NotGiven = NOT_GIVEN,
        card: payment_create_params.Card | NotGiven = NOT_GIVEN,
        customer: payment_create_params.Customer | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        parent_payment_id: str | NotGiven = NOT_GIVEN,
        payment_method: Literal[
            "BASIC_CARD",
            "CRYPTO",
            "FLEXEPIN",
            "MACROPAY",
            "SKRILL",
            "PAYRETAILERS",
            "LOCALPAYMENT",
            "MONNET",
            "PAYPAL",
            "NETELLER",
            "TRUSTPAYMENTS",
            "PAYMAXIS",
            "GATE8TRANSACT",
            "TINK",
            "B2BINPAY",
            "CLICK",
            "MONETIX",
            "PERFECTMONEY",
            "VOLT",
            "KESSPAY",
            "BILLLINE",
            "NGENIUS",
            "ASTROPAY",
            "ALYCEPAY",
            "PIX",
            "BOLETO",
            "UPI",
            "PAYTM",
            "NETBANKING",
            "FINRAX",
            "SPOYNT",
            "XINPAY",
            "OMNIMATRIX",
            "DPOPAY",
            "EXTERNAL_HPP",
            "XANPAY",
            "INRPAY",
            "ARI10",
            "SOFORT",
            "GIROPAY",
            "PAYSAFECARD",
            "PAYSAFECASH",
            "OPEN_BANKING",
            "KLARNA",
            "SPEI",
            "PAYCASH",
            "RAPIPAGO",
            "PAGOFACIL",
            "RAPIDTRANSFER",
            "MOBILE_MONEY",
            "INTERAC",
            "INTERAC_ETO",
            "INTERAC_RTO",
            "INTERAC_ACH",
            "PICPAY",
            "MOLLIE",
            "TED",
            "ZIPAY",
            "PSE",
            "EFECTY",
            "BANKTRANSFER",
            "PEC",
            "OXXO",
            "WEBPAY",
            "PAGOEFECTIVO",
            "MIFINITY",
            "PAYPORT",
            "JETONCASH",
            "JETONWALLET",
            "NODA",
            "NODA_REVOLUT",
            "ALFAKIT",
            "PAYFUN",
            "EMANAT",
            "M10",
            "RUBPAY",
            "MONERCHY",
            "MUCHBETTER",
            "YAPILY",
            "INAI",
            "IMPS",
            "RTGS",
            "PAYID",
            "ZOTAPAY",
            "SBP",
            "P2P_CARD",
            "P2P_IBAN",
            "P2P_SBP",
            "P2P_MOBILE",
            "PUSH",
            "GATEIQ",
            "VIETTEL",
            "ZALO",
            "QR",
            "CUP",
            "CODI",
            "PAY2PLAY",
            "BKASH",
            "NAGAD",
            "ROCKET",
            "VIRTUAL_ACCOUNT",
            "MULTIBANCO",
            "BLIK",
            "MBWAY",
            "P24",
            "MISTERCASH",
            "MACH",
            "KHIPU",
            "NEFT",
            "STICPAY",
            "SBERPAY",
            "MOBILE_COMMERCE",
            "BINANCE_PAY",
            "MPAY",
            "CHEK",
            "KLAP_EFECTIVO",
            "KLAP_TRANSFERENCIA",
            "PAGO46",
            "HITES",
            "SERVIFACIL",
            "OPENPAYD",
            "FAWRY",
            "EPS",
            "IDEAL",
            "TRUSTLY",
            "USSD",
            "MPESA",
            "ENAIRA",
            "ONEVOUCHER",
            "BANCONTACT",
            "SWISH",
            "EFT",
            "GCASH",
            "PAYMAYA",
            "PAGO_MOVIL",
            "PAGO_MOVIL_INST",
            "BIOPAGO",
            "CASH",
            "VOUCHERRY",
            "APPLEPAY",
            "GOOGLEPAY",
            "BRITE",
            "VOUCHSTAR",
            "REVOLUT",
            "ONLINE_BANKING",
            "PROMPTPAY",
            "TRUEMONEY",
            "MOMOPAY_VN",
            "MOMOPAY_RW",
            "VNPAY_QR",
            "N26",
            "WISE",
            "PAYDO_WALLET",
            "PAPARA",
            "PAYOUT_SEPA_BATCH",
            "PAYOUT_NONSEPA_REQUEST",
        ]
        | NotGiven = NOT_GIVEN,
        recurring_token: str | NotGiven = NOT_GIVEN,
        reference_id: str | NotGiven = NOT_GIVEN,
        return_url: str | NotGiven = NOT_GIVEN,
        start_recurring: bool | NotGiven = NOT_GIVEN,
        subscription: payment_create_params.Subscription | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Payment:
        """
        Payment request, used for DEPOSITS, WITHDRAWALS and REFUNDS

        Args:
          currency: Payment currency

          payment_type: Payment Type

          additional_parameters: Additional parameters required by some payment providers. Contact support for
              more information.

          amount: Payment amount

          billing_address: Customer's billing address

          card: You must be PCI DSS compliant to collect card data on your side. If you are not
              certified, do not add this field to your request and we will collect the data on
              our page.

          description: Description of the transaction shown to the Customer. Can be sent outside the
              system.

          parent_payment_id: Id of initial deposit for refunds, Id of initial recurring payment for
              subsequent payments

          payment_method: Payment Method

          recurring_token: To continue recurring chain, send a token from a previously initiated recurring
              payment.

          reference_id: Reference assigned by Merchant. Will not go outside the system. Will be sent
              unchanged in the PaymentResponse.

          return_url: URL to redirect Customer after processing

          start_recurring: Send 'true' if you want this payment to initiate recurring chain. Default is
              'false'.

          subscription: Subscription to bill customers at regular intervals. Used only with
              'startRecurring=true'.

          webhook_url: Url to receive payment status notifications

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/payments",
            body=await async_maybe_transform(
                {
                    "currency": currency,
                    "payment_type": payment_type,
                    "additional_parameters": additional_parameters,
                    "amount": amount,
                    "billing_address": billing_address,
                    "card": card,
                    "customer": customer,
                    "description": description,
                    "parent_payment_id": parent_payment_id,
                    "payment_method": payment_method,
                    "recurring_token": recurring_token,
                    "reference_id": reference_id,
                    "return_url": return_url,
                    "start_recurring": start_recurring,
                    "subscription": subscription,
                    "webhook_url": webhook_url,
                },
                payment_create_params.PaymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Payment,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Payment:
        """
        Find Payment by Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/v1/payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Payment,
        )

    async def list(
        self,
        *,
        created: payment_list_params.Created | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        updated: payment_list_params.Updated | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PaymentListResponse:
        """
        Get a list of payments sorted by creation date (most recent first)

        Args:
          limit: The numbers of items to return. Default is 50.

          offset: The number of items to skip before starting to collect the result set. Default
              is 0.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/v1/payments",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "created": created,
                        "limit": limit,
                        "offset": offset,
                        "updated": updated,
                    },
                    payment_list_params.PaymentListParams,
                ),
            ),
            cast_to=PaymentListResponse,
        )


class PaymentsResourceWithRawResponse:
    def __init__(self, payments: PaymentsResource) -> None:
        self._payments = payments

        self.create = to_raw_response_wrapper(
            payments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            payments.retrieve,
        )
        self.list = to_raw_response_wrapper(
            payments.list,
        )

    @cached_property
    def operations(self) -> OperationsResourceWithRawResponse:
        return OperationsResourceWithRawResponse(self._payments.operations)


class AsyncPaymentsResourceWithRawResponse:
    def __init__(self, payments: AsyncPaymentsResource) -> None:
        self._payments = payments

        self.create = async_to_raw_response_wrapper(
            payments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            payments.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            payments.list,
        )

    @cached_property
    def operations(self) -> AsyncOperationsResourceWithRawResponse:
        return AsyncOperationsResourceWithRawResponse(self._payments.operations)


class PaymentsResourceWithStreamingResponse:
    def __init__(self, payments: PaymentsResource) -> None:
        self._payments = payments

        self.create = to_streamed_response_wrapper(
            payments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            payments.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            payments.list,
        )

    @cached_property
    def operations(self) -> OperationsResourceWithStreamingResponse:
        return OperationsResourceWithStreamingResponse(self._payments.operations)


class AsyncPaymentsResourceWithStreamingResponse:
    def __init__(self, payments: AsyncPaymentsResource) -> None:
        self._payments = payments

        self.create = async_to_streamed_response_wrapper(
            payments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            payments.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            payments.list,
        )

    @cached_property
    def operations(self) -> AsyncOperationsResourceWithStreamingResponse:
        return AsyncOperationsResourceWithStreamingResponse(self._payments.operations)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .payments import (
    PaymentsResource,
    AsyncPaymentsResource,
    PaymentsResourceWithRawResponse,
    AsyncPaymentsResourceWithRawResponse,
    PaymentsResourceWithStreamingResponse,
    AsyncPaymentsResourceWithStreamingResponse,
)
from .subscriptions import (
    SubscriptionsResource,
    AsyncSubscriptionsResource,
    SubscriptionsResourceWithRawResponse,
    AsyncSubscriptionsResourceWithRawResponse,
    SubscriptionsResourceWithStreamingResponse,
    AsyncSubscriptionsResourceWithStreamingResponse,
)

__all__ = [
    "PaymentsResource",
    "AsyncPaymentsResource",
    "PaymentsResourceWithRawResponse",
    "AsyncPaymentsResourceWithRawResponse",
    "PaymentsResourceWithStreamingResponse",
    "AsyncPaymentsResourceWithStreamingResponse",
    "SubscriptionsResource",
    "AsyncSubscriptionsResource",
    "SubscriptionsResourceWithRawResponse",
    "AsyncSubscriptionsResourceWithRawResponse",
    "SubscriptionsResourceWithStreamingResponse",
    "AsyncSubscriptionsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PaymentCreateParams", "BillingAddress", "Card", "Customer", "Subscription", "SubscriptionRetryStrategy"]


class PaymentCreateParams(TypedDict, total=False):
    currency: Required[str]
    """Payment currency"""

    payment_type: Required[Annotated[Literal["DEPOSIT", "WITHDRAWAL", "REFUND"], PropertyInfo(alias="paymentType")]]
    """Payment Type"""

    additional_parameters: Annotated[Dict[str, str], PropertyInfo(alias="additionalParameters")]
    """Additional parameters required by some payment providers.

    Contact support for more information.
    """

    amount: float
    """Payment amount"""

    billing_address: Annotated[BillingAddress, PropertyInfo(alias="billingAddress")]
    """Customer's billing address"""

    card: Card
    """You must be PCI DSS compliant to collect card data on your side.

    If you are not certified, do not add this field to your request and we will
    collect the data on our page.
    """

    customer: Customer

    description: str
    """Description of the transaction shown to the Customer.

    Can be sent outside the system.
    """

    parent_payment_id: Annotated[str, PropertyInfo(alias="parentPaymentId")]
    """
    Id of initial deposit for refunds, Id of initial recurring payment for
    subsequent payments
    """

    payment_method: Annotated[
        Literal[
            "BASIC_CARD",
            "CRYPTO",
            "FLEXEPIN",
            "MACROPAY",
            "SKRILL",
            "PAYRETAILERS",
            "LOCALPAYMENT",
            "MONNET",
            "PAYPAL",
            "NETELLER",
            "TRUSTPAYMENTS",
            "PAYMAXIS",
            "GATE8TRANSACT",
            "TINK",
            "B2BINPAY",
            "CLICK",
            "MONETIX",
            "PERFECTMONEY",
            "VOLT",
            "KESSPAY",
            "BILLLINE",
            "NGENIUS",
            "ASTROPAY",
            "ALYCEPAY",
            "PIX",
            "BOLETO",
            "UPI",
            "PAYTM",
            "NETBANKING",
            "FINRAX",
            "SPOYNT",
            "XINPAY",
            "OMNIMATRIX",
            "DPOPAY",
            "EXTERNAL_HPP",
            "XANPAY",
            "INRPAY",
            "ARI10",
            "SOFORT",
            "GIROPAY",
            "PAYSAFECARD",
            "PAYSAFECASH",
            "OPEN_BANKING",
            "KLARNA",
            "SPEI",
            "PAYCASH",
            "RAPIPAGO",
            "PAGOFACIL",
            "RAPIDTRANSFER",
            "MOBILE_MONEY",
            "INTERAC",
            "INTERAC_ETO",
            "INTERAC_RTO",
            "INTERAC_ACH",
            "PICPAY",
            "MOLLIE",
            "TED",
            "ZIPAY",
            "PSE",
            "EFECTY",
            "BANKTRANSFER",
            "PEC",
            "OXXO",
            "WEBPAY",
            "PAGOEFECTIVO",
            "MIFINITY",
            "PAYPORT",
            "JETONCASH",
            "JETONWALLET",
            "NODA",
            "NODA_REVOLUT",
            "ALFAKIT",
            "PAYFUN",
            "EMANAT",
            "M10",
            "RUBPAY",
            "MONERCHY",
            "MUCHBETTER",
            "YAPILY",
            "INAI",
            "IMPS",
            "RTGS",
            "PAYID",
            "ZOTAPAY",
            "SBP",
            "P2P_CARD",
            "P2P_IBAN",
            "P2P_SBP",
            "P2P_MOBILE",
            "PUSH",
            "GATEIQ",
            "VIETTEL",
            "ZALO",
            "QR",
            "CUP",
            "CODI",
            "PAY2PLAY",
            "BKASH",
            "NAGAD",
            "ROCKET",
            "VIRTUAL_ACCOUNT",
            "MULTIBANCO",
            "BLIK",
            "MBWAY",
            "P24",
            "MISTERCASH",
            "MACH",
            "KHIPU",
            "NEFT",
            "STICPAY",
            "SBERPAY",
            "MOBILE_COMMERCE",
            "BINANCE_PAY",
            "MPAY",
            "CHEK",
            "KLAP_EFECTIVO",
            "KLAP_TRANSFERENCIA",
            "PAGO46",
            "HITES",
            "SERVIFACIL",
            "OPENPAYD",
            "FAWRY",
            "EPS",
            "IDEAL",
            "TRUSTLY",
            "USSD",
            "MPESA",
            "ENAIRA",
            "ONEVOUCHER",
            "BANCONTACT",
            "SWISH",
            "EFT",
            "GCASH",
            "PAYMAYA",
            "PAGO_MOVIL",
            "PAGO_MOVIL_INST",
            "BIOPAGO",
            "CASH",
            "VOUCHERRY",
            "APPLEPAY",
            "GOOGLEPAY",
            "BRITE",
            "VOUCHSTAR",
            "REVOLUT",
            "ONLINE_BANKING",
            "PROMPTPAY",
            "TRUEMONEY",
            "MOMOPAY_VN",
            "MOMOPAY_RW",
            "VNPAY_QR",
            "N26",
            "WISE",
            "PAYDO_WALLET",
            "PAPARA",
            "PAYOUT_SEPA_BATCH",
            "PAYOUT_NONSEPA_REQUEST",
        ],
        PropertyInfo(alias="paymentMethod"),
    ]
    """Payment Method"""

    recurring_token: Annotated[str, PropertyInfo(alias="recurringToken")]
    """
    To continue recurring chain, send a token from a previously initiated recurring
    payment.
    """

    reference_id: Annotated[str, PropertyInfo(alias="referenceId")]
    """Reference assigned by Merchant.

    Will not go outside the system. Will be sent unchanged in the PaymentResponse.
    """

    return_url: Annotated[str, PropertyInfo(alias="returnUrl")]
    """URL to redirect Customer after processing"""

    start_recurring: Annotated[bool, PropertyInfo(alias="startRecurring")]
    """Send 'true' if you want this payment to initiate recurring chain.

    Default is 'false'.
    """

    subscription: Subscription
    """Subscription to bill customers at regular intervals.

    Used only with 'startRecurring=true'.
    """

    webhook_url: Annotated[str, PropertyInfo(alias="webhookUrl")]
    """Url to receive payment status notifications"""


class BillingAddress(TypedDict, total=False):
    address_line1: Annotated[str, PropertyInfo(alias="addressLine1")]
    """Line 1 of the address (e.g., Number, street, etc)"""

    address_line2: Annotated[str, PropertyInfo(alias="addressLine2")]
    """Line 2 of the address (e.g., Suite, apt)"""

    city: str
    """City name"""

    country_code: Annotated[str, PropertyInfo(alias="countryCode")]
    """2-character IS0-3166-1 country code"""

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """Postal code"""

    state: str
    """State code"""


class Card(TypedDict, total=False):
    cardholder_name: Annotated[str, PropertyInfo(alias="cardholderName")]
    """Cardholder's name printed on the card"""

    card_number: Annotated[str, PropertyInfo(alias="cardNumber")]
    """Card primary account number (PAN). All non-numeric characters will be ignored."""

    card_security_code: Annotated[str, PropertyInfo(alias="cardSecurityCode")]
    """Card security code (CVV2 / CVC2 / CAV2)"""

    expiry_month: Annotated[str, PropertyInfo(alias="expiryMonth")]
    """Card expiration month, 2 digits"""

    expiry_year: Annotated[str, PropertyInfo(alias="expiryYear")]
    """Card expiration year, 4 digits"""


class Customer(TypedDict, total=False):
    account_name: Annotated[str, PropertyInfo(alias="accountName")]
    """Customer account name in the provider's system.

    Used for some types of withdrawals.
    """

    account_number: Annotated[str, PropertyInfo(alias="accountNumber")]
    """Customer account number in the provider's system.

    Used for some types of withdrawals.
    """

    bank: str
    """Customer bank. Used for some types of withdrawals."""

    bank_branch: Annotated[str, PropertyInfo(alias="bankBranch")]
    """Customer bank branch. Used for some types of withdrawals."""

    citizenship_country_code: Annotated[str, PropertyInfo(alias="citizenshipCountryCode")]
    """Customer country of citizenship"""

    date_of_birth: Annotated[str, PropertyInfo(alias="dateOfBirth")]

    date_of_first_deposit: Annotated[str, PropertyInfo(alias="dateOfFirstDeposit")]
    """Date of the first deposit from the customer"""

    deposits_amount: Annotated[int, PropertyInfo(alias="depositsAmount")]
    """How much the customer has deposited, in base currency"""

    deposits_cnt: Annotated[int, PropertyInfo(alias="depositsCnt")]
    """How many times the customer made a deposit"""

    document_number: Annotated[str, PropertyInfo(alias="documentNumber")]
    """An identifier for the customer assigned by a government authority"""

    document_type: Annotated[
        Literal[
            "AR_CDI",
            "AR_CUIL",
            "AR_CUIT",
            "AR_DNI",
            "AR_OTRO",
            "BR_CNPJ",
            "BR_CPF",
            "CL_OTRO",
            "CL_RUN",
            "CL_RUT",
            "CO_CC",
            "CO_CE",
            "CO_DL",
            "CO_DNI",
            "CO_NE",
            "CO_NIT",
            "CO_PP",
            "CO_SS",
            "CO_TI",
            "CR_CDI",
            "EC_DNI",
            "EC_PP",
            "EC_RUC",
            "GT_CUI",
            "GT_DPI",
            "GT_NIT",
            "MX_CURP",
            "MX_IFE",
            "MX_PP",
            "MX_RFC",
            "PA_CIP",
            "PE_CE",
            "PE_DNI",
            "PE_OTRO",
            "PE_PP",
            "PE_RUC",
        ],
        PropertyInfo(alias="documentType"),
    ]
    """Document Type"""

    email: str
    """Email address of the customer"""

    first_name: Annotated[str, PropertyInfo(alias="firstName")]

    kyc_status: Annotated[bool, PropertyInfo(alias="kycStatus")]
    """Indicates whether the customer has passed KYC verification"""

    last_name: Annotated[str, PropertyInfo(alias="lastName")]

    locale: str
    """Customer preferred display language"""

    payment_instrument_kyc_status: Annotated[bool, PropertyInfo(alias="paymentInstrumentKycStatus")]
    """
    Indicates whether the payment instrument (usually the card number) has passed
    KYC verification
    """

    phone: str
    """International phone number of the customer, without the '+'.

    Use a space as a separator between the dialing country code and local phone
    number.
    """

    reference_id: Annotated[str, PropertyInfo(alias="referenceId")]
    """Id of the customer assigned by Merchant"""

    routing_group: Annotated[str, PropertyInfo(alias="routingGroup")]
    """Identify the customer as belonging to a specific group that is used for routing"""

    withdrawals_amount: Annotated[int, PropertyInfo(alias="withdrawalsAmount")]
    """How much the customer has withdrawn, in base currency"""

    withdrawals_cnt: Annotated[int, PropertyInfo(alias="withdrawalsCnt")]
    """How many times the customer made a withdrawal"""


class SubscriptionRetryStrategy(TypedDict, total=False):
    frequency: Required[int]
    """
    The number of intervals after which the system will retry the payment after an
    unsuccessful attempt
    """

    number_of_cycles: Required[Annotated[int, PropertyInfo(alias="numberOfCycles")]]
    """Required number of retries"""

    amount_adjustments: Annotated[Iterable[int], PropertyInfo(alias="amountAdjustments")]
    """
    If specified, the nth element contains the percentage of the initial amount that
    will be charged for the nth retry
    """

    frequency_unit: Annotated[Literal["MINUTE", "DAY", "WEEK", "MONTH"], PropertyInfo(alias="frequencyUnit")]
    """The interval at which the subscription is retried.

    Use 'MINUTE' for testing purposes only.
    """


class Subscription(TypedDict, total=False):
    frequency: Required[int]
    """The number of intervals after which a subscriber is billed.

    For example, if the frequencyUnit is DAY with an frequency of 2, the
    subscription is billed once every two days.
    """

    amount: float
    """The amount to be used for subsequent payments.

    If not specified, the amount of the original payment is used.
    """

    description: str
    """Description for subsequent recurring payments"""

    frequency_unit: Annotated[Literal["MINUTE", "DAY", "WEEK", "MONTH"], PropertyInfo(alias="frequencyUnit")]
    """The interval at which the subscription is billed.

    Use 'MINUTE' for testing purposes only.
    """

    number_of_cycles: Annotated[int, PropertyInfo(alias="numberOfCycles")]
    """Required number of subsequent recurring payments.

    Unlimited if value is not specified.
    """

    retry_strategy: Annotated[SubscriptionRetryStrategy, PropertyInfo(alias="retryStrategy")]
    """Retry strategy for subscription.

    If not specified, the subscription is canceled after the first failed payment
    attempt.
    """

    start_time: Annotated[str, PropertyInfo(alias="startTime")]
    """Date and time of the 1st cycle.

    if not specified, then calculated as (initialDeposit.createTime +
    frequency\\**frequencyUnit).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .payment import Payment as Payment
from .subscription import Subscription as Subscription
from .payment_list_params import PaymentListParams as PaymentListParams
from .payment_create_params import PaymentCreateParams as PaymentCreateParams
from .payment_list_response import PaymentListResponse as PaymentListResponse
from .subscription_update_params import SubscriptionUpdateParams as SubscriptionUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from paymaxis import Paymaxis, AsyncPaymaxis
from tests.utils import assert_matches_type
from paymaxis.types import Payment, PaymentListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPayments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Paymaxis) -> None:
        payment = client.payments.create(
            currency="EUR",
            payment_type="DEPOSIT",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Paymaxis) -> None:
        payment = client.payments.create(
            currency="EUR",
            payment_type="DEPOSIT",
            additional_parameters={
                "bankCode": "ABHY0065032",
                "countryOfBirth": "CY",
            },
            amount=1e-18,
            billing_address={
                "address_line1": "7, Sunny street",
                "address_line2": "Office 3",
                "city": "Limassol",
                "country_code": "CY",
                "postal_code": "4141",
                "state": "CA",
            },
            card={
                "cardholder_name": "John Smith",
                "card_number": "4000 0000 0000 0002",
                "card_security_code": "010",
                "expiry_month": "01",
                "expiry_year": "2030",
            },
            customer={
                "account_name": "accountName",
                "account_number": "accountNumber",
                "bank": "bank",
                "bank_branch": "bankBranch",
                "citizenship_country_code": "AU",
                "date_of_birth": "2001-12-03",
                "date_of_first_deposit": "2021-01-01",
                "deposits_amount": 5000,
                "deposits_cnt": 5000,
                "document_number": "documentNumber",
                "document_type": "AR_CDI",
                "email": "my@email.com",
                "first_name": "John",
                "kyc_status": True,
                "last_name": "Smith",
                "locale": "ru",
                "payment_instrument_kyc_status": True,
                "phone": "357 123123123",
                "reference_id": "customer_123",
                "routing_group": "VIP",
                "withdrawals_amount": 1000,
                "withdrawals_cnt": 1000,
            },
            description="Deposit 123 via TEST shop",
            parent_payment_id="91d27876e87f4b22b3ecd53924bf973d",
            payment_method="BASIC_CARD",
            recurring_token="recurringToken",
            reference_id="payment_id=123;custom_ref=456",
            return_url="https://mywebsite.com/{id}/{referenceId}/{state}/{type}",
            start_recurring=True,
            subscription={
                "frequency": 2,
                "amount": 99.99,
                "description": "Subscription to service",
                "frequency_unit": "MINUTE",
                "number_of_cycles": 12,
                "retry_strategy": {
                    "frequency": 2,
                    "number_of_cycles": 12,
                    "amount_adjustments": [1, 1, 1],
                    "frequency_unit": "MINUTE",
                },
                "start_time": "2030-12-25T10:11:12",
            },
            webhook_url="https://mywebsite.com/webhooks",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Paymaxis) -> None:
        response = client.payments.with_raw_response.create(
            currency="EUR",
            payment_type="DEPOSIT",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Paymaxis) -> None:
        with client.payments.with_streaming_response.create(
            currency="EUR",
            payment_type="DEPOSIT",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Paymaxis) -> None:
        payment = client.payments.retrieve(
            "id",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Paymaxis) -> None:
        response = client.payments.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Paymaxis) -> None:
        with client.payments.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Paymaxis) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.payments.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: Paymaxis) -> None:
        payment = client.payments.list()
        assert_matches_type(PaymentListResponse, payment, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Paymaxis) -> None:
        payment = client.payments.list(
            created={
                "gte": "2021-10-13T10:26:18",
                "lt": "2021-10-13T10:39:34",
            },
            limit=1,
            offset=0,
            updated={
                "gte": "2021-10-13T10:26:18",
                "lt": "2021-10-13T10:39:34",
            },
        )
        assert_matches_type(PaymentListResponse, payment, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Paymaxis) -> None:
        response = client.payments.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(PaymentListResponse, payment, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Paymaxis) -> None:
        with client.payments.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(PaymentListResponse, payment, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncPayments:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncPaymaxis) -> None:
        payment = await async_client.payments.create(
            currency="EUR",
            payment_type="DEPOSIT",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncPaymaxis) -> None:
        payment = await async_client.payments.create(
            currency="EUR",
            payment_type="DEPOSIT",
            additional_parameters={
                "bankCode": "ABHY0065032",
                "countryOfBirth": "CY",
            },
            amount=1e-18,
            billing_address={
                "address_line1": "7, Sunny street",
                "address_line2": "Office 3",
                "city": "Limassol",
                "country_code": "CY",
                "postal_code": "4141",
                "state": "CA",
            },
            card={
                "cardholder_name": "John Smith",
                "card_number": "4000 0000 0000 0002",
                "card_security_code": "010",
                "expiry_month": "01",
                "expiry_year": "2030",
            },
            customer={
                "account_name": "accountName",
                "account_number": "accountNumber",
                "bank": "bank",
                "bank_branch": "bankBranch",
                "citizenship_country_code": "AU",
                "date_of_birth": "2001-12-03",
                "date_of_first_deposit": "2021-01-01",
                "deposits_amount": 5000,
                "deposits_cnt": 5000,
                "document_number": "documentNumber",
                "document_type": "AR_CDI",
                "email": "my@email.com",
                "first_name": "John",
                "kyc_status": True,
                "last_name": "Smith",
                "locale": "ru",
                "payment_instrument_kyc_status": True,
                "phone": "357 123123123",
                "reference_id": "customer_123",
                "routing_group": "VIP",
                "withdrawals_amount": 1000,
                "withdrawals_cnt": 1000,
            },
            description="Deposit 123 via TEST shop",
            parent_payment_id="91d27876e87f4b22b3ecd53924bf973d",
            payment_method="BASIC_CARD",
            recurring_token="recurringToken",
            reference_id="payment_id=123;custom_ref=456",
            return_url="https://mywebsite.com/{id}/{referenceId}/{state}/{type}",
            start_recurring=True,
            subscription={
                "frequency": 2,
                "amount": 99.99,
                "description": "Subscription to service",
                "frequency_unit": "MINUTE",
                "number_of_cycles": 12,
                "retry_strategy": {
                    "frequency": 2,
                    "number_of_cycles": 12,
                    "amount_adjustments": [1, 1, 1],
                    "frequency_unit": "MINUTE",
                },
                "start_time": "2030-12-25T10:11:12",
            },
            webhook_url="https://mywebsite.com/webhooks",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPaymaxis) -> None:
        response = await async_client.payments.with_raw_response.create(
            currency="EUR",
            payment_type="DEPOSIT",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPaymaxis) -> None:
        async with async_client.payments.with_streaming_response.create(
            currency="EUR",
            payment_type="DEPOSIT",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncPaymaxis) -> None:
        payment = await async_client.payments.retrieve(
            "id",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncPaymaxis) -> None:
        response = await async_client.payments.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncPaymaxis) -> None:
        async with async_client.payments.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncPaymaxis) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.payments.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncPaymaxis) -> None:
        payment = await async_client.payments.list()
        assert_matches_type(PaymentListResponse, payment, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncPaymaxis) -> None:
        payment = await async_client.payments.list(
            created={
                "gte": "2021-10-13T10:26:18",
                "lt": "2021-10-13T10:39:34",
            },
            limit=1,
            offset=0,
            updated={
                "gte": "2021-10-13T10:26:18",
                "lt": "2021-10-13T10:39:34",
            },
        )
        assert_matches_type(PaymentListResponse, payment, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncPaymaxis) -> None:
        response = await async_client.payments.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(PaymentListResponse, payment, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncPaymaxis) -> None:
        async with async_client.payments.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(PaymentListResponse, payment, path=["response"])

        assert cast(Any, response.is_closed) is True
